#Ruby
#'camera2vue.rb'
#
#This script will export the current SketchUP view to a python script which creates a matching camera in Vue
#
#Version 1.0
#Author: Alex Jenyon, based heavily on 'camera2modo' by Grant Marshall

require 'sketchup.rb'

def camera2vue

 UI.messagebox "Please ensure that your Model Info>Units are set to Decimal Meters and Precision is set to maximum. 
 (Cancel the next dialog and change these settings if necessary)."

 model = Sketchup.active_model
 view = model.active_view
 camera = view.camera

 value = UI.openpanel("Export Camera to Python script", "", "SU_camera_2_vue.py")

 if value
 camscript = File.new(value.to_s, "w")

#Write python script - this section selects the main camera in Vue
camscript.puts("DeselectAll()")
camscript.puts("SelectByType(VuePython.VOT_Camera)")
camscript.puts("cam = GetSelectedObjectByIndex(0)")

model = Sketchup.active_model
 view = model.active_view
 camera = view.camera


# Get camera eyepoint
 eye = camera.eye
 vx = eye.x
 vy = eye.y
 vz = eye.z

 # Get camera direction
 direction = camera.direction
 dx = direction.x
 dy = direction.y
 dz = direction.z

 # Get camera up vector
 up = camera.up

 # Get camera field of view
 fov = camera.fov

 # Calculate X-rotation of camera
 rx = (Math.atan2(dz,Math.sqrt(dx*dx+dy*dy)))
 rx2 = (Math.atan2(dz,Math.sqrt(dx*dx+dy*dy)).radians)
 #convert orientation for Vue
 new_rx = 90-rx2

 # Calculate Z-rotation of camera
 rz = Math.atan2(dx,dy)
 rz2 = Math.atan2(dx,dy).radians
 #convert orientation for Vue
 new_rz = 180-rz2

 # Calculate Y-rotation of camera (roll)
 #Method:  Transform the global 'y' vector so that it aligns with aligns with the local z-axis
 #The roll angle will then be the angle between the resulting vector and the camera direction

 #Create transform that applies 'x' rotation
 p1 = Geom::Point3d.new 0,0,0
 v1 = Geom::Vector3d.new 1,0,0
 angle = rx
 rotate_x = Geom::Transformation.rotation p1, v1, angle

 #Apply it to the global 'y' vector
 vector = [0,0,1]
 vector2 = vector.transform rotate_x

 #Create transform that applies 'z' rotation
 p2 = Geom::Point3d.new 0,0,0
 v2 = Geom::Vector3d.new 0,0,-1
 angle2 = rz
 rotate_z = Geom::Transformation.rotation p2, v2, angle2

 #Apply it to the result of the last transform
 vector3 = vector2.transform rotate_z

 #Compare the result to the local y-axis and find the angle between the two
 y_local = camera.up
 cosine_roll = y_local.dot vector3

 #Clamp the cosine result to -1, 1: due to rounding errors it may be very slightly out of range
  if cosine_roll>1
  cosine_roll = 1
  else
  cosine_roll = cosine_roll
  end
  if cosine_roll<-1
  cosine_roll = -1
  else
  cosine_roll = cosine_roll
  end

 #Work out the angle between the two vectors in degrees
 ry = Math.acos(cosine_roll)
 ry2 = ry.radians

 #Work out if the rotation is clockwise or ccw
 ccw = camera.xaxis.z
 if ccw>0
 ry2 = -ry2
 else
 ry2 = ry2
 end

#write camera data
camscript.puts("cam.SetPosition(" + vx.to_f.to_m.to_s + ", " + vy.to_f.to_m.to_s + ", " + vz.to_f.to_m.to_s + ")")
camscript.puts("cam.SetRotationAngles(" + new_rx.to_s + ", 0, " + new_rz.to_s + ")")
camscript.puts("cam.Rotate(0, 0, " + ry2.to_s + ", true)")

 end

 #close file (python script)
 camscript.close

end

# Add menu item 
if (not file_loaded?("camera2vue.rb"))
 #add_separator_to_menu("Camera")
 UI.menu("Camera").add_item("Current view to Vue") { camera2vue }
end

file_loaded("camera2vue.rb")

